package conf

import (
	"backend/logging"
	"github.com/BurntSushi/toml"
	"log"
	"path/filepath"
	"strings"
	"time"
)

type Logging struct {
	FileName     string        `toml:"filename"`
	RollSize     int64         `toml:"roll_size"`
	RollInterval time.Duration `toml:"roll_interval"`
	Level        string        `toml:"level"`
	PanicOnFatal bool          `toml:"panic_on_fatal"`
}

type Config struct {
	Web    string  `toml:"web"`
	Address    string  `toml:"address"`
	SessionKey string  `toml:"session_key"`
	SessionVal string  `toml:"session_val"`
	SessionTTL int64   `toml:"session_ttl"`
	Logging    Logging `toml:"logging"`
}

func Init(cfg string) *Config {
	var res Config
	_, err := toml.DecodeFile(cfg, &res)
	if err != nil {
		log.Panicf("can't decode configuration file: %s", err.Error())
	}
	return &res
}

func (c *Config) str2Level() int {
	l := strings.ToLower(c.Logging.Level)
	switch l {
	case "info":
		return logging.INFO
	case "debug":
		return logging.DEBUG
	case "warn":
		return logging.WARN
	case "error":
		return logging.ERROR
	case "fatal":
		return logging.FATAL
	default:
		panic("invalid level " + c.Logging.Level)
	}
}

func (c *Config) GetLogger(cwd string) *logging.Logger {
	return &logging.Logger{
		RollSize:     c.Logging.RollSize * (1 << 20),
		RollInterval: c.Logging.RollInterval * time.Hour,
		FileName:     filepath.Join(cwd, c.Logging.FileName),
		Level:        c.str2Level(),
		PanicOnFatal: c.Logging.PanicOnFatal,
	}
}
