/***********************************************
        File Name: auth
        Author: Abby Cin
        Mail: abbytsing@gmail.com
        Created Time: 10/31/19 7:52 PM
***********************************************/

package middleware

import (
	"backend/session"
	"github.com/gin-gonic/gin"
	"net/http"
	"strconv"
)

type Auth struct {
	mgr  *session.SessionManager
	excl map[string][]string
}

func NewAuth(mgr *session.SessionManager) *Auth {
	r := &Auth{
		mgr:  mgr,
	}
	return r
}

func (a *Auth) Serve(c *gin.Context) {
	ss := a.mgr.StartSession(c.Writer, c.Request)
	status := ss.Get(a.mgr.SessionKey())
	id := ss.Get(ss.Id())

	if status != a.mgr.SessionVal() || id == "" {
		c.JSON(http.StatusUnauthorized, gin.H{
			"code":  -1,
			"error": "unauthorized",
		})
		c.Abort()
		return
	}

	if c.Request.Method == http.MethodPut || c.Request.Method == http.MethodPost {
		n := c.Request.Header.Get("Content-Length")
		r, e := strconv.ParseInt(n, 10, 64)
		if e != nil {
			c.JSON(http.StatusBadRequest, gin.H{
				"code":  -1,
				"error": "invaild content length",
			})
			c.Abort()
			return
		}
		if r > (10 << 20) {
			c.JSON(http.StatusBadRequest, gin.H{
				"code":  -1,
				"error": "body too large",
			})
			c.Abort()
			return
		}
	}
	c.Next()
}
