package service

import (
	"encoding/json"
	"errors"
	"github.com/gin-gonic/gin"
	"io/ioutil"
	"net/http"
	"os"
	"path"
	"sync"
)

type DBManage struct {
	dbcfg string
	mtx sync.Mutex
}

func NewDBManage() *DBManage {
	return &DBManage{
		dbcfg: "",
		mtx:   sync.Mutex{},
	}
}

func (d *DBManage) Serve(c *gin.Context) {
	if c.Request.Method == http.MethodPut {
		d.HandleUpdate(c)
	} else if c.Request.Method == http.MethodGet {
		d.HandleGet(c)
	} else {
		newError(c, -1, "404 not found")
	}
}

func (d *DBManage) update(c *gin.Context) error {
	if _, err := os.Stat(d.dbcfg); os.IsNotExist(err) {
		return errors.New("please make sure you have set server root correctly")
	}
	data, err := ioutil.ReadAll(c.Request.Body)
	if err != nil {
		return err
	}

	j := make(map[string]interface{})
	err = json.Unmarshal(data, &j)
	if err == nil {
		tmp, err := json.MarshalIndent(j, "", " ")
		if err == nil {
			data = tmp
		}
	}
	f, err := os.OpenFile(d.dbcfg, os.O_WRONLY | os.O_TRUNC, 00644)
	if err != nil {
		return err
	}

	defer f.Close()

	_, err = f.Write(data)
	return err
}

func (d *DBManage) UpdateServerRoot(root string) {
	d.dbcfg = path.Join(root, "config", "tophttpserver.database.json")
}

func (d *DBManage) HandleUpdate(c *gin.Context) {
	d.mtx.Lock()
	defer d.mtx.Unlock()

	err := d.update(c)
	if err != nil {
		newError(c, -1, err.Error())
		return
	}
	newError(c, 0, "ok")
}

func (d *DBManage) get(c *gin.Context) (map[string]interface{}, error) {
	if _, err := os.Stat(d.dbcfg); os.IsNotExist(err) {
		return nil, errors.New("please make sure you have set server root correctly")
	}
	data, err := ioutil.ReadFile(d.dbcfg)
	if err != nil {
		return nil, err
	}
	m := make(map[string]interface{})
	if len(data) == 0 {
		data = []byte("{}")
	}
	err = json.Unmarshal(skipBom(data), &m)
	if err != nil {
		return nil, err
	}
	return m, nil
}

func (d *DBManage) HandleGet(c *gin.Context) {
	d.mtx.Lock()
	defer d.mtx.Unlock()

	m, err := d.get(c)
	if err != nil {
		newError(c, -1, err.Error())
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code": 0,
		"data": m,
		"msg": "ok",
	})
}